/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-10-17 16:31:14.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.print.service.impl;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.core.print.common.jpa.OrderBuilder;
import cn.com.qmth.examcloud.core.print.common.jpa.SearchBuilder;
import cn.com.qmth.examcloud.core.print.common.jpa.SpecUtils;
import cn.com.qmth.examcloud.core.print.common.jpa.SqlWrapper;
import cn.com.qmth.examcloud.core.print.common.utils.Check;
import cn.com.qmth.examcloud.core.print.entity.PrintingProject;
import cn.com.qmth.examcloud.core.print.repository.PrintingProjectRepository;
import cn.com.qmth.examcloud.core.print.service.PrintingProjectService;
import cn.com.qmth.examcloud.core.print.service.bean.PrintingProjectConvert;
import cn.com.qmth.examcloud.core.print.service.bean.PrintingProjectInfo;
import cn.com.qmth.examcloud.core.print.service.bean.PrintingProjectQuery;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;

import java.util.List;

import static cn.com.qmth.examcloud.core.print.common.Constants.PRT_CODE_500;

/**
 * 印刷项目相关接口
 *
 * @author: fengdesheng
 * @since: 2018/10/17
 */
@Service
public class PrintingProjectServiceImpl implements PrintingProjectService {
    @Autowired
    private PrintingProjectRepository printingProjectRepository;
    @Autowired
    private JdbcTemplate jdbcTemplate;

    @Override
    public Page<PrintingProjectInfo> getPrintingProjectList(PrintingProjectQuery query) {
        Check.isNull(query, "查询参数不能为空！");
        //查询条件
        SearchBuilder searches = new SearchBuilder();
        if (query.getOrgId() != null) {
            searches.eq("orgId", query.getOrgId());
        }
        if (query.getExamId() != null) {
            searches.eq("examId", query.getExamId());
        }
        if (query.getPmId() != null) {
            searches.eq("pmId", query.getPmId());
        }
        if (query.getSupplierId() != null) {
            searches.eq("supplierId", query.getSupplierId());
        }
        Specification<PrintingProject> spec = SpecUtils.buildSearchers(PrintingProject.class, searches.build());
        //排序条件
        OrderBuilder orders = new OrderBuilder().desc("id");
        //分页条件
        Pageable pageable = SpecUtils.buildPageable(query.getPageNo(), query.getPageSize(), orders.build());
        Page<PrintingProject> page = printingProjectRepository.findAll(spec, pageable);
        return PrintingProjectConvert.ofPage(page);
    }

    @Override
    public PrintingProjectInfo getPrintingProjectById(Long id) {
        Check.isNull(id, "印刷项目ID不能为空！");
        PrintingProject project = printingProjectRepository.findOne(id);
        if (project == null) {
            throw new StatusException(PRT_CODE_500, "印刷项目信息不存在！");
        }
        return PrintingProjectConvert.of(project);
    }

    @Override
    public void updatePrintingProject(PrintingProjectInfo info) {
        Check.isNull(info.getId(), "印刷项目ID不能为空！");
        Check.isNull(info.getSupplierId(), "供应商ID不能为空！");
        Check.isBlank(info.getSupplierName(), "供应商名称不能为空！");
        Check.isNull(info.getPmId(), "项目经理ID不能为空！");
        Check.isBlank(info.getPmName(), "项目经理名称不能为空！");
        Check.isNull(info.getPrepareStartTime(), "准备开始时间不能为空！");
        Check.isNull(info.getPrepareEndTime(), "准备结束时间不能为空！");
        Check.isNull(info.getPrintStartTime(), "印刷开始时间不能为空！");
        Check.isNull(info.getPrintEndTime(), "印刷结束时间不能为空！");
        Check.isNull(info.getMailStartTime(), "邮寄开始时间不能为空！");
        Check.isNull(info.getMailEndTime(), "邮寄结束时间不能为空！");

        PrintingProject entity = printingProjectRepository.findOne(info.getId());
        if (entity == null) {
            throw new StatusException(PRT_CODE_500, "印刷项目信息不存在！");
        }

        entity.setSupplierId(info.getSupplierId());
        entity.setSupplierName(info.getSupplierName());
        entity.setPmId(info.getPmId());
        entity.setPmName(info.getPmName());
        entity.setPrepareStartTime(info.getPrepareStartTime());
        entity.setPrepareEndTime(info.getPrepareEndTime());
        entity.setPrintStartTime(info.getPrintStartTime());
        entity.setPrintEndTime(info.getPrintEndTime());
        entity.setMailStartTime(info.getMailStartTime());
        entity.setMailEndTime(info.getMailEndTime());
        printingProjectRepository.save(entity);
    }

    @Override
    public void syncPrintingProject(PrintingProjectInfo info) {
        Check.isNull(info.getOrgId(), "学校ID不能为空！");
        Check.isBlank(info.getOrgName(), "学校名称不能为空！");
        Check.isNull(info.getExamId(), "考试ID不能为空！");
        Check.isBlank(info.getExamName(), "考试名称不能为空！");

        SearchBuilder searches = new SearchBuilder()
                .eq("orgId", info.getOrgId())
                .eq("examId", info.getExamId());
        Specification<PrintingProject> spec = SpecUtils.buildSearchers(PrintingProject.class, searches.build());

        List<PrintingProject> projects = printingProjectRepository.findAll(spec);
        if (projects == null || projects.isEmpty()) {
            //新增信息
            PrintingProject project = PrintingProjectConvert.of(info);
            printingProjectRepository.save(project);
            return;
        }

        //更新信息
        for (PrintingProject project : projects) {
            project.setOrgName(info.getOrgName());
            project.setExamName(info.getExamName());
        }
        printingProjectRepository.save(projects);
    }

    @Override
    public void dataInit() {
        SqlWrapper wrapper = new SqlWrapper()
                .select("em.id as examId,em.name as examName,em.root_org_id as orgId,org.name as orgName")
                .from("ec_e_exam").as("em")
                .innerJoin("ec_b_org").as("org").on("org.id", "em.root_org_id")
                .where().eq("em.exam_type", "TRADITION");
        List<PrintingProject> list = jdbcTemplate.query(wrapper.build(), new BeanPropertyRowMapper(PrintingProject.class));
        printingProjectRepository.save(list);
    }

}